<?php
/**
 * (C) 2025 Motive Commerce Search Corp S.L. <info@motive.co>
 *
 * This file is part of Motive Commerce Search.
 *
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Motive (motive.co)
 * @copyright (C) 2025 Motive Commerce Search Corp S.L. <info@motive.co>
 * @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 */

namespace Motive\Prestashop\Builder\Price;

use Motive\Prestashop\Model\Price;
use Motive\Prestashop\Model\XResultPrice;

if (!defined('_PS_VERSION_')) {
    exit;
}

class FastPriceBuilder extends BasePriceBuilder
{
    /** @var int */
    protected $id_customer;

    /** @var int */
    protected $id_group;

    /** @var int */
    protected $id_currency;

    /** @var int */
    protected $id_country;

    /** @var int */
    protected $id_state;

    /** @var int */
    protected $zipcode;

    /**
     * FastPriceBuilder constructor.
     *
     * @param \Context $context
     * @param int $decimals
     * @param int $id_customer
     * @param int $id_group
     * @param int $id_address
     * @param int $id_country
     * @param int $id_state
     * @param string $zipcode
     */
    public function __construct(\Context $context, $decimals = 6, $id_customer = 0, $id_group = 0, $id_address = 0, $id_country = 0, $id_state = 0, $zipcode = 0)
    {
        parent::__construct($context, $decimals);

        if ($id_address || !$id_country || !$id_state || !$zipcode) {
            $address = \Address::initialize($id_address, true);
            $id_country = (int) $address->id_country;
            $id_state = (int) $address->id_state;
            $zipcode = $address->postcode;
        }

        if (!$id_customer && \Validate::isLoadedObject($context->customer)) {
            $id_customer = $context->customer->id;
        }

        if ($id_customer) {
            $id_group = \Customer::getDefaultGroupId((int) $id_customer);
        }
        if (!$id_group) {
            $id_group = (int) \Group::getCurrent()->id;
        }

        $this->id_customer = $id_customer;
        $this->id_group = $id_group;
        $this->id_currency = \Validate::isLoadedObject($context->currency) ? (int) $context->currency->id : (int) \Configuration::get('PS_CURRENCY_DEFAULT');
        $this->id_country = $id_country;
        $this->id_state = $id_state;
        $this->zipcode = $zipcode;
    }

    public function compute($productRow, $idProductAttribute = null)
    {
        $idProduct = $productRow['p_id'];
        if (!$idProductAttribute) {
            $idProductAttribute = (int) $productRow['p_default_attribute'];
        }

        $regular = $this->calc($idProduct, $idProductAttribute, false);
        $on_sale = !$regular ? $regular : $this->calc($idProduct, $idProductAttribute);

        return Price::build($regular, $on_sale);
    }

    public function computeResult($productRow, $idProductAttribute = null)
    {
        $idProduct = $productRow['p_id'];
        $regular = $this->calc($idProduct, $idProductAttribute, false);
        $on_sale = !$regular ? $regular : $this->calc($idProduct, $idProductAttribute);
        $alternativePrice = $this->calc($idProduct, $idProductAttribute, true, !$this->useTax);

        return XResultPrice::build($regular, $on_sale, $this->useTax, $alternativePrice);
    }

    /**
     * Calculate the price of a product with various parameters.
     *
     * @param int $idProduct the ID of the product
     * @param int $idProductAttribute the ID of the product attribute
     * @param bool $usereduc whether to apply reductions/discounts (default: true)
     * @param bool $useTax whether to use tax (default: value of $this->useTax)
     * @param int $quantity the quantity of the product (default: 1)
     *
     * @return float the calculated price of the product
     */
    public function calc($idProduct, $idProductAttribute, $usereduc = true, $useTax = null, $quantity = 1)
    {
        if ($useTax === null) {
            $useTax = $this->useTax;
        }
        $specific_price_output = null;

        return \Product::priceCalculation(
            $this->context->shop->id,
            $idProduct,
            $idProductAttribute,
            $this->id_country,
            $this->id_state,
            $this->zipcode,
            $this->id_currency,
            $this->id_group,
            $quantity,
            $useTax,
            $this->decimals,
            /* $only_reduc */ false,
            $usereduc,
            /* $with_ecotax */ true,
            $specific_price_output,
            /* $use_group_reduction */ true,
            $this->id_customer
        );
    }
}
