<?php
/**
 * (C) 2025 Motive Commerce Search Corp S.L. <info@motive.co>
 *
 * This file is part of Motive Commerce Search.
 *
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Motive (motive.co)
 * @copyright (C) 2025 Motive Commerce Search Corp S.L. <info@motive.co>
 * @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 */

namespace Motive\Prestashop\Builder;

use Motive\Prestashop\Builder\Price\BasePriceBuilder as PriceBuilder;
use Motive\Prestashop\Config;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ShopperPriceBuilder
{
    /** @var \Context */
    protected $context;

    /** @var PriceBuilder */
    protected $priceBuilder;

    /**
     * SchemaBuilder constructor.
     *
     * @param \context $context
     */
    public function __construct($context)
    {
        $this->context = $context;
        $decimals = version_compare(_PS_VERSION_, '1.7.7.0', '>=') ?
            $this->context->getComputingPrecision() :
            (int) \Configuration::get('PS_PRICE_DISPLAY_PRECISION');
        $priceBuilderClass = Config::getShopperPricesPriceBuilder();
        $this->priceBuilder = new $priceBuilderClass($context, $decimals);
    }

    /**
     * Enrich products with price field.
     *
     * @param array $data partial of product list
     *
     * @return array same object with prices
     */
    public function enrich(array $products)
    {
        if (!$this->priceBuilder->getShowPrice()) {
            // No data
            return [];
        }

        $ids = [];
        foreach ($products as $key => $product) {
            if (empty($product['id']) || (int) $product['id'] <= 0) {
                unset($products[$key]);
                continue;
            }
            $ids[] = (int) $product['id'];
        }

        if (empty($ids)) {
            return [];
        }

        $this->priceBuilder->prefetchIds($ids);
        $productsInfo = $this->getProductsInfo($ids);
        $productsInfo = array_combine(array_column($productsInfo, 'id'), $productsInfo);

        foreach ($products as &$product) {
            // If product does not exist
            if (empty($productsInfo[$product['id']])) {
                continue;
            }

            $info = $productsInfo[$product['id']];

            // If not available_for_order or show_price = 0, skip
            if (empty($info['p_available_for_order']) && empty($info['p_show_price'])) {
                continue;
            }
            $product['price'] = $this->priceBuilder->getResult($info);
            $product['availability'] = [
                'minimumQuantity' => max(1, (int) $info['p_minimal_quantity']),
            ];
            if (empty($product['variants'])) {
                continue;
            }
            foreach ($product['variants'] as &$variant) {
                if (empty($variant['id'])) {
                    continue;
                }
                $variant['price'] = $this->priceBuilder->getResult($info, (int) $variant['id']);
            }
        }

        return $products;
    }

    /**
     * Get product info by product IDs.
     *
     * @return array of product data
     */
    protected function getProductsInfo($ids)
    {
        $ps_ = _DB_PREFIX_;
        $ids = implode(',', $ids);
        $sql = "
            SELECT
                ps.id_product AS id,
                ps.id_product AS p_id,
                ps.show_price AS p_show_price,
                ps.price AS p_base_price,
                ps.id_tax_rules_group AS p_id_tax_rules_group,
                ps.available_for_order AS p_available_for_order,
                ps.minimal_quantity AS p_minimal_quantity,
                p.cache_default_attribute AS p_default_attribute
            FROM {$ps_}product AS p
              JOIN {$ps_}product_shop AS ps
                  ON (ps.id_product = p.id_product AND ps.id_shop = {$this->context->shop->id})
            WHERE p.id_product IN ($ids)
            ORDER BY ps.id_product
        ";

        return empty($ids) ? [] : \Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
    }
}
